@@ -11,15 +11,21 @@ to the native file dialogues available in Tk 4.2 and newer, and the
 directory dialogue available in Tk 8.3 and newer.
 These interfaces were written by Fredrik Lundh, May 1997.
 """
+__all__ = ["FileDialog", "LoadFileDialog", "SaveFileDialog",
+           "Open", "SaveAs", "Directory",
+           "askopenfilename", "asksaveasfilename", "askopenfilenames",
+           "askopenfile", "askopenfiles", "asksaveasfile", "askdirectory"]
 
-from tkinter import *
+import fnmatch
+import os
+from tkinter import (
+    Frame, LEFT, YES, BOTTOM, Entry, TOP, Button, Tk, X,
+    Toplevel, RIGHT, Y, END, Listbox, BOTH, Scrollbar,
+)
 from tkinter.dialog import Dialog
 from tkinter import commondialog
 from tkinter.simpledialog import _setup_dialog
 
-import os
-import fnmatch
-
 
 dialogstates = {}
 
@@ -455,7 +461,6 @@ def test():
 
     # Start off with UTF-8
     enc = "utf-8"
-    import sys
 
     # See whether CODESET is defined
     try:
@@ -471,9 +476,9 @@ def test():
     try:
         fp=open(openfilename,"r")
         fp.close()
-    except:
+    except BaseException as exc:
         print("Could not open File: ")
-        print(sys.exc_info()[1])
+        print(exc)
 
     print("open", openfilename.encode(enc))
 
