@@ -23,7 +23,6 @@
 
 #ifndef PYSQLITE_CURSOR_H
 #define PYSQLITE_CURSOR_H
-#define PY_SSIZE_T_CLEAN
 #include "Python.h"
 
 #include "statement.h"
@@ -42,19 +41,12 @@ typedef struct
     PyObject* row_factory;
     pysqlite_Statement* statement;
     int closed;
-    int reset;
     int locked;
     int initialized;
 
-    /* the next row to be returned, NULL if no next row available */
-    PyObject* next_row;
-
     PyObject* in_weakreflist; /* List of weak references */
 } pysqlite_Cursor;
 
-extern PyTypeObject *pysqlite_CursorType;
-
 int pysqlite_cursor_setup_types(PyObject *module);
 
-#define UNKNOWN (-1)
 #endif
